#include <stdint.h>
#include <stdio.h>
#include <string.h>


uint8_t Crc8Lut[256];

uint8_t crc8_init(uint8_t poly) {
    for (uint16_t i = 0; i < 256; i++) {
        uint8_t crc = (uint8_t)i;

        for (uint8_t j = 1; j <= 8; j++) {
            if (crc & 0x80) {
                crc = (uint8_t)((crc << 1) ^ poly);
            } else {
                crc <<= 1;
            }
        }

        Crc8Lut[i] = crc;
    }
}

uint8_t crc8(uint8_t* data, uint8_t length) {
    uint8_t crc = 0;
    while (length--) {
        crc ^= *data++;
        crc = Crc8Lut[crc];
    }
    return crc;
}


void main(int argc, char *argv[]) {
    crc8_init(0x2F);

    printf("const uint8_t Crc8Lut[] = { ");
    for (uint16_t i = 0; i < 256; i++) {
        if ((i > 0) && ((i % 16) == 0)) { printf("\n                            "); }
        printf("0x%02X, ", Crc8Lut[i]);
    }
    printf("};\n\n");

    uint8_t* text = "123456789";
    uint8_t crc = crc8(text, strlen(text));
    printf("Hello 0x%02X\n", crc);
}
