#include <stdint.h>
#include <stdio.h>
#include <string.h>


uint8_t Crc8LutL[16];
uint8_t Crc8LutH[16];

uint8_t crc8_init(uint8_t poly) {
    for (uint8_t i = 0; i < 16; i++) {
        uint8_t crc = (uint8_t)i;

        for (uint8_t j = 1; j <= 8; j++) {
            if (crc & 0x80) {
                crc = (uint8_t)((crc << 1) ^ poly);
            } else {
                crc <<= 1;
            }
        }

        Crc8LutL[i] = crc;
    }

    for (uint8_t i = 0; i < 16; i++) {
        uint8_t crc = (uint8_t)i << 4;

        for (uint8_t j = 1; j <= 8; j++) {
            if (crc & 0x80) {
                crc = (uint8_t)((crc << 1) ^ poly);
            } else {
                crc <<= 1;
            }
        }

        Crc8LutH[i] = crc;
    }
}

uint8_t crc8(uint8_t* data, uint8_t length) {
    uint8_t crc = 0;
    while (length--) {
        crc ^= *data++;
        crc = Crc8LutL[crc & 0x0F] ^ Crc8LutH[crc >> 4];
    }
    return crc;
}


void main(int argc, char *argv[]) {
    crc8_init(0x2F);

    printf("const uint8_t Crc8LutL[] = { ");
    for (uint8_t i = 0; i < 16; i++) {
        printf("0x%02X, ", Crc8LutL[i]);
    }
    printf("};\n");

    printf("const uint8_t Crc8LutH[] = { ");
    for (uint8_t i = 0; i < 16; i++) {
        printf("0x%02X, ", Crc8LutH[i]);
    }
    printf("};\n\n");

    uint8_t* text = "123456789";
    uint8_t crc = crc8(text, strlen(text));
    printf("Hello 0x%02X\n", crc);
}
